﻿using DetourFunc;
using System;
using System.Runtime.InteropServices;

public delegate void SleepDelegate(int milliseconds);

class Program
{
    [DllImport("kernel32.dll", EntryPoint = "Sleep")]
    public static extern void KernelSleep(int milliseconds);

    static void Main(string[] _)
    {
        using (var item = MethodReplacer.Win32FuncWithManagedFunc<SleepDelegate>(KernelSleep, (milliseconds) =>
                        {
                            Console.WriteLine("DllImport called: " + milliseconds);
                            _org_Sleep?.Invoke(milliseconds);
                        }, out _org_Sleep))
        {
            Console.WriteLine(DateTime.Now + ": before - Sleep");
            KernelSleep(1 * 1000);
            Console.WriteLine(DateTime.Now + ": after - Sleep");

            Console.WriteLine();
        }

        using (var managed = MethodReplacer.Win32FuncWithManagedFunc<SleepDelegate>("kernel32.dll", "Sleep", ManagedSleep, out _org_Sleep))
        {
            Console.WriteLine(DateTime.Now + ": before - Sleep");
            KernelSleep(1 * 1000);
            Console.WriteLine(DateTime.Now + ": after - Sleep");
        }

        Console.WriteLine();

        using (MethodReplacer.Win32FuncWithManagedFunc<SleepDelegate>("kernel32.dll", "Sleep", (milliseconds) =>
        {
            Console.WriteLine("AnonymousSleep called: " + milliseconds);
            _org_Managed?.Invoke(milliseconds);
        }, out _org_Managed))
        {
            Console.WriteLine(DateTime.Now + ": before - Sleep");
            KernelSleep(1 * 1000);
            Console.WriteLine(DateTime.Now + ": after - Sleep");
        }

        //Console.WriteLine();

        //using (MethodReplacer.Win32FuncWithManagedFunc<Action<int>>("kernel32.dll", "Sleep", (milliseconds) =>
        //{
        //    Console.WriteLine("You can't see this message because NOT SUPPORTED FOR Generic Type");
        //    _org_Action?.Invoke(milliseconds);
        //}, out _org_Action))
        //{
        //}

        {
            Console.WriteLine();
            Console.WriteLine("Press any key to exit...");
            Console.ReadLine();
        }
    }

    // static Action<int> _org_Action;

    static SleepDelegate _org_Managed;

    static SleepDelegate _org_Sleep;
    public static void ManagedSleep(int milliseconds)
    {
        Console.WriteLine("ManagedSleep called: " + milliseconds);
        _org_Sleep?.Invoke(milliseconds);
    }

}
